/****************************************************************************
** Meta object code from reading C++ file 'seqdriver.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.14)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "seqdriver.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'seqdriver.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.14. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SeqDriver_t {
    QByteArrayData data[10];
    char stringdata0[95];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SeqDriver_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SeqDriver_t qt_meta_stringdata_SeqDriver = {
    {
QT_MOC_LITERAL(0, 0, 9), // "SeqDriver"
QT_MOC_LITERAL(1, 10, 9), // "midiEvent"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 16), // "snd_seq_event_t*"
QT_MOC_LITERAL(4, 38, 2), // "ev"
QT_MOC_LITERAL(5, 41, 10), // "procEvents"
QT_MOC_LITERAL(6, 52, 19), // "setDiscardUnmatched"
QT_MOC_LITERAL(7, 72, 2), // "on"
QT_MOC_LITERAL(8, 75, 16), // "setPortUnmatched"
QT_MOC_LITERAL(9, 92, 2) // "id"

    },
    "SeqDriver\0midiEvent\0\0snd_seq_event_t*\0"
    "ev\0procEvents\0setDiscardUnmatched\0on\0"
    "setPortUnmatched\0id"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SeqDriver[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   34,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   37,    2, 0x0a /* Public */,
       6,    1,   38,    2, 0x0a /* Public */,
       8,    1,   41,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Int,    9,

       0        // eod
};

void SeqDriver::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SeqDriver *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->midiEvent((*reinterpret_cast< snd_seq_event_t*(*)>(_a[1]))); break;
        case 1: _t->procEvents(); break;
        case 2: _t->setDiscardUnmatched((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->setPortUnmatched((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SeqDriver::*)(snd_seq_event_t * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SeqDriver::midiEvent)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SeqDriver::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SeqDriver.data,
    qt_meta_data_SeqDriver,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SeqDriver::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SeqDriver::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SeqDriver.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int SeqDriver::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void SeqDriver::midiEvent(snd_seq_event_t * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
